#! /usr/bin/python


__author__="hanis (Jan Rychtar)"


import getopt
import sys


from gsmirroring.site_controller import SiteController


def usage():
    print """
    usage: python gsmirror.py options

    options:
        -s, --site <STRING>       the webspace name of your site (e.g. mySite)
        -d, --domain <STRING>     domain of your Google Apps hosted domain (e.g. example.com)
        -p, --path <STRING>       path where is (will be) the root directory (e.g. /home/me/sites/)
        -n, --name <STRING>       name of the root directory where the site is to be saved (e.g. my_site)
        -t, --templates <STRING>  path to the directory with template files
        -l  --log                 print progress of the mirroring process
        -h, --help                print help for gsmirror
            --email <STRING>      the user's email address or username
            --password <STRING>   the password for the user's account

    examples:
        python gsmirror.py --site=mysite --path=/home/me/sites/ --name=my_site --templates=/home/me/teplates/
             Run the mirroring of the site 'mysite' found in
             the URL http://sites.google.com/site/mysite/ and
             save it into the directory 'my_site' at
             the path '/home/me/sites/' using templates
             at the path '/home/me/teplates/'
        python gsmirror.py -s mysite2 -d mydomain.org -p /home/me/ -n my_site2 -l
             Run the mirroring of the site 'mysite2' hosted on
             a Google Apps domain 'mydomain.org' found in
             the URL http://sites.google.com/a/mydomain.org/mysite2/ and
             save it into the directory 'my_site2' at
             the path '/home/me/' using default templates
             at the path './templates/default_templates/' and during the mirroring will
             print information about a progress.

    templates:
        If you use your own templates (i.e. templates option is specified) then
        there (at the path you specified) have to be five files named as follows:
            'file_cabinet_template.tmpl'
            'announcements_page_template.tmpl'
            'announcement_template.tmpl'
            'listpage_template.tmpl'
            'webpage_template.tmpl'
        If you don't specify templates, default templates will be used.
        Default templates consist of the same five files as above and are stored
        in the directory './templates/default_templates/'.
        For more information about templates check out the readme.txt file or
        default templates.


    path & name:
        These two options determine where the site is to be saved.
        PATH indicate the desired path at the disk where is (will be created)
        a directory named NAME that is supposed to be the root directory in which
        the site will be saved.
        PATH should be an abslolute path and has to end with '/'
        (e.g. /home/john/my_sites/).
        NAME sholud somehow describe the site and has to have the format of the
        direcotry name (e.g. cars_for_sale).
        If NAME isn't specified, the SITE option (site name) is used.
        If PATH isn't specified, the current working directory ('./') is used.

    For more information check out the readme.txt file
    """


def main():

    try:
        opts, args = getopt.getopt(sys.argv[1:], 'hs:d:p:n:t:l', ['site=',
                    'domain=', 'path=', 'name=', 'templates=', 'email=',
                    'password=', 'log', 'help'])

    except getopt.error, msg:
        print str(msg)
        usage()
        sys.exit(2)

    site = None
    domain = None
    path = None
    name = None
    templates = None
    log = False
    email = None
    password = None

    for option, arg in opts:
        if option in ('--help', '-h'):
            usage()
            sys.exit()
        elif option in ('--site', '-s'):
            site = arg
        elif option in ('--domain', '-d'):
            domain = arg
        elif option in ('--path', '-p'):
            path = arg
        elif option in ('--name', '-n'):
            name = arg
        elif option in ('--templates', '-t'):
            templates = arg
        elif option == '--email':
            email = arg
        elif option == '--password':
            password = arg
        elif option in ('--log', '-l'):
            log = True

    if not site:
        print 'site argument is missing'
        usage()
        sys.exit(2)

    if not name:
        name=site

    if not path:
        path='./'



    siteController = SiteController(site=site, domain=domain, progression=log,
                    email=email, password=password, templates=templates)

    siteController.process_modification_doc(path=path, directory=name)
    if siteController.modified_since_lasttime():
        site = siteController.get_site()
        siteController.save_site_to_disk(site=site, path=path, directory=name)


if __name__ == "__main__":
    main()


 


"""
This module provides several classes that represent Google Sites entities.
Classes are used for storing data and the subsequent usage in templates.
"""

__author__="hanis (Jan Rychtar)"


import time
import calendar


PAGE_NAME = 'index.html'



class Author():
    def __init__(self, name, email):
        self.email = email
        self.name = name


class CollectiveObject:
    """
    Represents attributes thet are common to all objects and defines methods for
    setting the attributes
    """
    revision = None
    updated = None
    author = None
    def set_author(self, author_name, author_email):
        self.author = Author(author_name, author_email)
    def set_date_and_revision(self, updated, revision):
        self.updated = Date(updated)
        self.revision = revision


class Commentable():
    """
    Classes that represents entities which may store comments inherit a method
    for adding comments from this class.
    """
    def add_comment(self, text, author_name, author_email, updated, revision):        
        self.comments.append(Comment(text, author_name, author_email, updated, revision))


class Parent():
    """
    Classes that represents entities which may store subpages
    (e.g. webpages) inherit a method for adding subpages from this class.
    """
    def add_child(self, child):
        self.childs.append(child)


class Attachmentable():
    """
    Classes that represents entities which may store attachments or 
    webattachments inherit from methods for adding attachments and 
    webattachments this class.
    """
    def add_attachment(self, id, etag, author_name, author_email, name, summary, updated, revision, save):
        self.attachments.append(Attachment(id, author_name, author_email, name, summary, updated, revision, etag=etag, save=save))
    def add_web_attachment(self, id, author_name, author_email, name, summary, updated, revision, web_src):
        self.web_attachments.append(Attachment(id, author_name, author_email, name, summary, updated, revision, web_src))



class Site(Parent, Attachmentable):
    """
    Represents the root object that store all entities of the site
    """
    def __init__(self):
        self.childs = []
        self.attachments = []
        self.path=''
        self.path_to_root=''
        self.landing_page_id = 0
        self.landing=False

    def get_landing_page(self):
        for child in self.childs:
            if child.landing:
                return child

    def get_non_landing_pages(self):
        childs = []
        for child in self.childs:
            if not child.landing:
                childs.append(child)
        return childs


class LandingPage():
    def __init__(self, id, pagename, path, rel_path):
        self.id=id
        self.pagename=pagename
        self.path=path
        self.rel_path=rel_path        



class Page(CollectiveObject, Commentable, Parent, Attachmentable):
    """
    Represents webpages, file cabinets, announcement pages, announcements anad 
    list pages
    """
    def __init__(self, id, title, pagename, kind, author_name, author_email,
                    updated, revision, parent, content=None,
                    embedded_content=None, etag=None, landing=False):
        self.id = id
        self.etag = etag
        self.kind = kind
        self.set_author(author_name, author_email)
        self.set_date_and_revision(updated, revision)
        self.title = title
        self.content = content
        self.embedded_content = embedded_content #only for announcemects
        self.comments = [] #list of comments
        self.childs = [] #list of subpages
        self.attachments = [] #list of attachments
        self.web_attachments = [] #only for file cabinets
        self.parent = parent
        self.pagename = pagename
        self.landing = landing
        #self.subpage_path = './' + self.pagename + '/' + PAGE_NAME
        self.path=self.parent.path + self.pagename + '/'
        self.path_to_root=parent.path_to_root + '../'
        self.list_items=None #only for list pages        

    def get_path_to_root(self):
        if self.landing:
            return './'
        else:           
            return self.path_to_root


    def get_predecessors(self):
        """
        Returs the list of all predecessors (except the object representing the
        site) sorted by distance from the page in increasing order
        """
        predecessors = []
        page = self
        while True:
            page = page.parent
            if isinstance(page, Site):
                break
            predecessors.append(page)
        predecessors.reverse()
        return predecessors


    def get_announcements(self):
        """
        Returns list of subpages that represents announcements
        """
        announcements = []
        for page in self.childs:
            if page.kind == 'announcement':
                announcements.append(page)
        return announcements



    def get_alternative_path_to(self, page):
        if self.landing:
            if page.landing:
                alt = './' + PAGE_NAME
            else:
                alt = page.path_to_root + PAGE_NAME
        elif page.landing:
            alt = self.path + PAGE_NAME
        else:
            alt = page.path_to_root + self.path + PAGE_NAME
        return alt




class Comment(CollectiveObject):
    def __init__(self, text, author_name, author_email, updated, revision):
        self.set_author(author_name, author_email)
        self.text = text
        self.set_date_and_revision(updated, revision)



class Attachment(CollectiveObject):
    def __init__(self, id, author_name, author_email, name, summary, updated, revision, web_src=None, etag=None, save=False):
        self.id = id
        self.etag = etag
        self.set_author(author_name, author_email)
        self.summary = summary
        self.name = name
        self.set_date_and_revision(updated, revision)
        self.link = './' + name
        self.web_src = web_src #only for web attachments
        self.save = save


class ListPage():
    def __init__(self):
        self.headers = []
        self.items = []
    def add_header(self, header):
        self.headers.append(header)
    def add_list_item(self, item):
        self.items.append(item)



class ListItem(CollectiveObject):
    def __init__(self, author_name, author_email, updated, revision):
        self.cells = []
        self.set_author(author_name, author_email)
        self.set_date_and_revision(updated, revision)
    def add_cell(self, cell):
        self.cells.append(cell)




class Date():
    """
    Google Sites API use the format ISO8601 for datetime of the form:
    yyyy-mm-ddThh:mm:ss.mmmZ
    Since such a format isn't suitable for web pages, this class allows
    the script to choose an arbitrary format (set in a template) for displaying
    datetime strings.
    """
    def __init__(self, date_string=None, unix_time=None):
        """
        Exactly one argument is admissible. 
        
        Args:
            date_string: String of the form "yyyy-mm-ddThh:mm:ss.mmmZ" (ISO8601).            
            unix_time: Number of seconds since the Unix Epoch 
                (January 1 1970 00:00:00 GMT)                    
        """
        if date_string:
            temp = date_string[0:date_string.find('.')]
            self.date = time.strptime(temp, '%Y-%m-%dT%H:%M:%S')
        elif unix_time:
            self.date = time.gmtime(unix_time)


    def get_unix_time(self):
        return calendar.timegm(self.date)



    def format(self, format=None):
        """            
            Args:
                format: String specifying the format of a datetime, where
                    String consists of directives (see python time module) and 
                    other additional letters.
            Returns: String representing a datetime of a form specified in 
            format argument
        """
        if format is None:
            format = '%B %d, %Y, %I:%M %p'
        return time.strftime(format, self.date)


    def get_current_unix_time(self):
        return time.time()


if __name__ == "__main__":
    pass



__author__="hanis (Jan Rychtar)"


from Cheetah.Template import Template
import gdata.sites.client
import gdata.sites.data
import gdata.gauth
from BeautifulSoup import BeautifulSoup


import time
import re
import os
import shutil


from objects import Site
from objects import Page
from objects import ListPage
from objects import ListItem
from objects import Date
from objects import LandingPage


from xml_module import ETagDocument



FEED_FETCH_STEP = 300

SOURCE = 'hanis-gsmirror-v1'

DEFAULT_TEMPLATES = './templates/default_templates/'

TMPL_FILE_CABINET = 'file_cabinet_template.tmpl'
TMPL_ANNOUNCEMENTS_PAGE = 'announcements_page_template.tmpl'
TMPL_ANNOUNCEMENT = 'announcement_template.tmpl'
TMPL_LISTPAGE = 'listpage_template.tmpl'
TMPL_WEBPAGE = 'webpage_template.tmpl'


PAGE_NAME = 'index.html'
ETAGS_DOC = 'etags.xml'

#------------possible kinds of content entries------------------------
ANNOUNCEMENT = 'announcement'
ANNOUNCEMENTS_PAGE = 'announcementspage'
ATTACHMENT = 'attachment'
COMMENT = 'comment'
FILE_CABINET = 'filecabinet'
LIST_ITEM = 'listitem'
LIST_PAGE = 'listpage'
WEBPAGE = 'webpage'
WEB_ATTACHMENT = 'webattachment'

PAGE_TYPES = [ANNOUNCEMENTS_PAGE, ANNOUNCEMENT, FILE_CABINET, WEBPAGE, LIST_PAGE]
#---------------------------------------------------------------------


class SiteController():
    """
    The key class that provides the functionality for connecting to the Google
    Sites, downloaading site content, processing content, applying templates
    and saving to the disk.
    """

    def __init__(self, site, domain=None, templates=None, progression = False, email=None, password=None):
        """
        Creates a new Site Controller
        
        Args:
            site: the webspace name of your site (e.g. mySite)
            domain: domain of your Google Apps hosted domain (e.g. example.com)
            templates: path to the directory with template files
            progression: if True, print progress of the mirroring process
            email: the user's email address or username
            password: the password for the user's account
        """

        if domain:
            self.client = gdata.sites.client.SitesClient(source=SOURCE, site=site, domain=domain)
        else:
            self.client = gdata.sites.client.SitesClient(source=SOURCE, site=site)

        self.progression = progression

        if templates:
            self.templates_path = templates
        else:
            self.templates_path = DEFAULT_TEMPLATES

        if password:
            self.client.client_login(email=email, password=password, source = self.client.source)

        self.etag_new = ETagDocument()
        self.etag_old = None
        self.landing_page = None



    def process_modification_doc(self, path, directory):
        """
        The method serves for checking whether or not the site has been mirrored
        before (ie. there is a file etags.xml at the path path/directory/ that
        contatins information about the last mirroring).
        In case this is the first mirroring of the site, the class attribute
        etag_old is set to None, which means that the mirroring process will
        crates a directory hierarchy and save all the current content of the
        site to the disk.
        In case the site has been mirrored before (to the same path),
        information about the last mirroring are set to etag_old (from
        path/directory/etags.xml), which means that the mirroring process will
        continue only if there has been some changes since the last time and
        will download and save only new or modified attachments.
        Args:
            path: path where is (will be) the root directory (e.g. /home/me/sites/)
            directory: name of the root directory where the site is to be saved (e.g. my_site)
        """
        path_to_etags = path + directory + '/' + ETAGS_DOC
        if os.access(path_to_etags, os.F_OK):
            self.etag_old = ETagDocument()
            self.etag_old.get_document(path_to_etags)
        self.etag_new.create()


    def modified_since_lasttime(self):
        """
        Returns: True if either it is the first mirroring of the site (etag_old
                    is set to None), or the sites has been modified since the
                    last time.
                 False if it's not the first mirroring and the site hasn't been
                 modified since the last time.
        """
        if self.etag_old:
            uri = '%s?max-results=%s' % (self.client.MakeActivityFeedUri(), 1)
            feed = self.client.GetActivityFeed(uri=uri)
            updated = Date(feed.entry[0].updated.text).get_unix_time()
            last_mirroring = self.etag_old.get_modification_time()
            return float(updated) > float(last_mirroring)
        else:
            return True



    def get_template(self, template_path):
        file_template = open(template_path, 'r')
        templateDef = file_template.read()
        file_template.close()
        return Template(templateDef)


    def save_as_file(self, content, path):
        outfile = file(path , 'w')
        outfile.write(content)
        outfile.close()


    def download_attachments(self):
        uri = '%s?kind=%s' % (self.client.make_content_feed_uri(), 'attachment')
        feed = self.client.GetContentFeed(uri=uri)
        for entry in feed.entry:
            self.client.DownloadAttachment(entry, OUTPUT_PATH + entry.title.text)



    def request_page_content(self, page_id):
        uri = '%s%s' % (self.client.make_content_feed_uri(), page_id)
        response = self.client.request(method='GET', uri=uri)
        content_str = str(response.read())
        return BeautifulSoup(content_str[content_str.find('<content'): content_str.find('</content>') + 10]).prettify()



    def get_site_path(self):
        if self.client.domain == 'site':
            path = 'http://sites.google.com/site/%s/' % (self.client.site)
        else:
            path = 'http://sites.google.com/a/%s/%s/' % (self.client.domain, self.client.site)
        return path

    def get_site_short_path(self):
        if self.client.domain == 'site':
            path = '/site/%s/' % (self.client.site)
        else:
            path = '/%s/%s/' % (self.client.domain, self.client.site)
        return path



    def parse_comment(self, comment):
        temp = comment[comment.find('<td'):comment.rfind('</td')]
        return temp[temp.find('>')+1:]



    def replace_paths(self, page_content, level, page):
        site_path = self.get_site_path()
        link_list = re.findall(r'<a href="'+ site_path +'.*?"', page_content)
        if self.landing_page.id == page.id:
            level = 0
        for link in link_list:
            att_test = link.find('?attredirects')            
            if att_test == -1:
                if link.endswith(self.landing_page.path, 0, -1):
                    relative_path = level * '../'
                    new_link = '<a href="%s%s"' % (relative_path, PAGE_NAME)
                else:
                    relative_path = level * '../' + link[len(site_path)+9:-1]
                    new_link = '<a href="%s/%s"' % (relative_path, PAGE_NAME)
            else:
                relative_path = level * '../' + link[len(site_path)+9:att_test]
                new_link = '<a href="%s"' % (relative_path)
            page_content = page_content.replace(link, new_link)

        #in one particular case there are links that don't start with htttp://sites/google ??!!
        #so this is considered as a temporary patch
        site_short_path = self.get_site_short_path()
        link_list = re.findall(r'<a href="'+ site_short_path +'.*?"', page_content)
        if self.landing_page.id == page.id:
            level = 0
        for link in link_list:
            att_test = link.find('?attredirects')
            if att_test == -1:
                if link.endswith(self.landing_page.path, 0, -1):
                    relative_path = level * '../'
                    new_link = '<a href="%s%s"' % (relative_path, PAGE_NAME)
                else:
                    relative_path = level * '../' + link[len(site_short_path)+9:-1]
                    new_link = '<a href="%s/%s"' % (relative_path, PAGE_NAME)
            else:
                relative_path = level * '../' + link[len(site_short_path)+9:att_test]
                new_link = '<a href="%s"' % (relative_path)
            page_content = page_content.replace(link, new_link)
        #-------------------------------------------------------------------------------
        return page_content



    def replace_attachment_source(self, page, attachment, parent, as_announcement=None):
        site_path = self.get_site_path()
        current_src='%s_/rsrc/[^>]*/%s%s' % (site_path, parent.path, attachment.name)
        src_list = re.findall(r''+current_src, page)        
        if as_announcement:
            new_src='./%s/%s' % (parent.pagename, attachment.name)
        else:
            new_src=attachment.link
        for src_item in src_list:
            page = page.replace(src_item, new_src)
        return page



    def save_site_to_disk(self, site, path, directory):
        """
        The method save the site to the disk to the path path/directory/.
        First, it removes attachments and pages that were saved last time and
        are not in the current version.

        Args:
            site: The output of the method get_site
            path: path where is (will be) the root directory (e.g. /home/me/sites/)
            directory: name of the root directory where the site is to be saved (e.g. my_site)
        """
        path_to_site = path + directory + '/'

        if self.etag_old:
            self.remove_unused(attachments=self.etag_old.get_unused_attachments(), pages=self.etag_old.get_unused_pages(), path=path_to_site)

        if not os.access(path_to_site, os.F_OK):
            os.mkdir(path_to_site)

        for child in site.childs:
            self.save_page_to_disk(page=child, path=path_to_site, site=site)

        self.etag_new.set_modification_time(str(time.time()))
        self.save_as_file(content=self.etag_new.to_string(), path=path_to_site + ETAGS_DOC)


    def remove_unused(self, attachments, pages, path):
        for attachment in attachments:
            if os.access(attachment[1], os.F_OK):
                if self.progression:
                    print 'removing unused attachment: ' + attachment[1]
                os.remove(attachment[1])
        for page in pages:
            abs_path = path + page[1]
            if os.access(abs_path, os.F_OK):
                if self.progression:
                    print 'removing unused page: ' + abs_path
                shutil.rmtree(abs_path)



    def save_page_to_disk(self, page, path, site):    
        full_path = path + page.path

        if not os.access(full_path, os.F_OK):
            if self.progression:
                print 'creating directory: ' + full_path
            os.mkdir(full_path)

        if self.progression:
            print 'saving page: ' + page.pagename

        self.etag_new.add_element(id=page.id,
                source=page.path, etag=page.etag, type=self.etag_new.PAGE)

        if page.kind == WEBPAGE:
            template_out = self.get_template(self.templates_path + TMPL_WEBPAGE)
        elif page.kind == FILE_CABINET:
            template_out = self.get_template(self.templates_path + TMPL_FILE_CABINET)
        elif page.kind == ANNOUNCEMENTS_PAGE:
            template_out = self.get_template(self.templates_path + TMPL_ANNOUNCEMENTS_PAGE)
        elif page.kind == ANNOUNCEMENT:
            template_out = self.get_template(self.templates_path + TMPL_ANNOUNCEMENT)
        elif page.kind == LIST_PAGE:
            template_out = self.get_template(self.templates_path + TMPL_LISTPAGE)
            
        template_out.page = page
        template_out.site = site

        if (page.id == self.landing_page.id):
            page_path = path
        else:
            page_path = full_path

        self.save_as_file(content=str(template_out), path=page_path + PAGE_NAME)

        for attachment in page.attachments:
            self.save_attachment_to_file(attachment, full_path)

        for child in page.childs:
            self.save_page_to_disk(child, path, site=site)



    def save_attachment_to_file(self, attachment, path):
        uri = '%s%s' % (self.client.MakeContentFeedUri(), attachment.id)
        att_entry = self.client.GetEntry(uri, desired_class=gdata.sites.data.ContentEntry)
        
        self.etag_new.add_element(id=attachment.id,
                source=path + attachment.name, etag=attachment.etag, type=self.etag_new.ATTACHMENT)
        if attachment.save:
            if self.progression:
                print 'saving attachment: ' + path + attachment.name
            self.client.DownloadAttachment(att_entry, path + attachment.name)
        elif self.progression:
            print 'omitting attachment: ' + path + attachment.name


    def get_landing_page(self):
        uri = '%s?path=/' % (self.client.MakeContentFeedUri())
        feed = self.client.GetContentFeed(uri=uri)
        entry = feed.entry[0]
        path=entry.GetAlternateLink().href
        rel_path = path[len(self.get_site_path()):]
        return LandingPage(id=entry.GetNodeId(), pagename=entry.page_name.text,
            path=entry.GetAlternateLink().href, rel_path=rel_path)


    def get_site(self):
        """
        This method downloads all the current site content (except of
        attachments) to the memory.

        Returns: objects.Site representing all the content of the mirrored site
        """
        site = Site()
        if self.progression:
            print 'downloading site'
        self.landing_page = self.get_landing_page()
        step = 0
        while True:
            uri = '%s?start-index=%s&max-results=%s' % (self.client.MakeContentFeedUri(), step*FEED_FETCH_STEP + 1, (FEED_FETCH_STEP - 1))
            feed = self.client.GetContentFeed(uri=uri)
            step+=1
            if not feed.entry: break
            for entry in feed.entry:
                if entry.FindParentLink() is None and entry.Kind() in PAGE_TYPES:
                    self.get_page(entry, level=1, parent=site)
        return site



    def get_site_content(self, level=1, parent=None):
        if self.progression:
            print 'downloading ocontent of: ' + parent.pagename
        step = 0
        while True:
            uri = '%s?parent=%s&start-index=%s&max-results=%s' % (self.client.MakeContentFeedUri(),
                        parent.id, step*FEED_FETCH_STEP + 1, (FEED_FETCH_STEP - 1))
            feed = self.client.GetContentFeed(uri=uri)
            step+=1
            if not feed.entry: break
            for entry in feed.entry:
                kind = entry.Kind()
                if kind in PAGE_TYPES:
                    self.get_page(entry, level, parent)
                if kind == ATTACHMENT:
                    self.get_attachment(entry, parent=parent)
                elif kind == COMMENT:
                    self.get_comment(entry, parent)
                elif kind == LIST_ITEM:
                    self.get_list_item(entry, parent)
                elif kind == WEB_ATTACHMENT:
                    self.get_web_attachment(entry, parent)


    def get_page(self, entry, level, parent):
        if self.progression:
            print 'downloading page: ' + entry.page_name.text
        landing = (entry.GetNodeId() == self.landing_page.id)

        page = Page(id=entry.GetNodeId(),
                    kind=entry.Kind(),
                    title=entry.title.text,
                    author_name=entry.author[0].name.text,
                    author_email=entry.author[0].email.text,
                    updated=entry.updated.text,
                    revision=entry.revision.text,
                    pagename=entry.page_name.text,
                    parent=parent,
                    etag=entry.etag,
                    landing=landing)

        if self.etag_old:
            self.etag_old.check_page(page.id, page.path)

        if page.kind == LIST_PAGE:
            list_page = ListPage()
            for header in entry.data.column:
                list_page.add_header(header.name)
            page.list_items = list_page

        content_str = self.request_page_content(entry.GetNodeId())
        self.get_site_content(level=level+1, parent=page)


        if page.landing:
            for attachment in page.attachments:
                attachment.link = './' + self.landing_page.rel_path + '/' + attachment.name
                

        if page.kind == ANNOUNCEMENT:
            content_str2 = content_str
            content_str2 = self.replace_paths(page_content=content_str2, level=level-1, page=page)
            for attachment in page.attachments:
                content_str2 = self.replace_attachment_source(page=content_str2, attachment=attachment, parent=page, as_announcement=True)
            page.embedded_content = content_str2

        content_str = self.replace_paths(page_content=content_str, level=level, page=page)
        for attachment in page.attachments:
            content_str = self.replace_attachment_source(page=content_str, attachment=attachment, parent=page)
        page.content = content_str
        parent.add_child(page)



    def get_list_item(self, entry, parent):
        if self.progression:
            print 'downloading list item at: ' + parent.pagename
        item = ListItem(author_name=entry.author[0].name.text,
                        author_email=entry.author[0].email.text,
                        updated=entry.updated.text,
                        revision=entry.revision.text)

        for cell in entry.field:
            item.add_cell(cell.text)
        parent.list_items.add_list_item(item)


    def get_attachment(self, entry, parent):
        if self.progression:
            print 'downloading attachment information: ' + entry.title.text
        if self.etag_old:
            save = self.etag_old.check_attachment(id=entry.GetNodeId(), etag=entry.etag)
        else:
            save = True
        parent.add_attachment(id=entry.GetNodeId(),
                                etag=entry.etag,
                                author_name=entry.author[0].name.text,
                                author_email=entry.author[0].email.text,
                                name=entry.title.text,
                                summary=entry.summary.text,
                                updated=entry.updated.text,
                                revision=entry.revision.text,
                                save = save)


    def get_web_attachment(self, entry, parent):
        if self.progression:
            print 'downloading webattachment: ' + entry.title.text
        parent.add_web_attachment(id=entry.GetNodeId(),
                                author_name=entry.author[0].name.text,
                                author_email=entry.author[0].email.text,
                                name=entry.title.text,
                                summary=entry.summary.text,
                                updated=entry.updated.text,
                                revision=entry.revision.text,
                                web_src=str(entry.content.src))



    def get_comment(self, entry, parent):
        if self.progression:
            print 'downloading comment at: ' + parent.pagename
        parent.add_comment(text=self.parse_comment(self.request_page_content(entry.GetNodeId())),
                            author_name=entry.author[0].name.text,
                            author_email=entry.author[0].email.text,
                            updated=entry.updated.text,
                            revision=entry.revision.text)




if __name__ == "__main__":
    pass






__author__="hanis (Jan Rychtar)"


from xml.dom import minidom


class ETagDocument():
    """
         Representation of an XML file that contains information about
         the state of the site elements such as timestamp of the last
         mirroring, list of attachments and list of pages (where each
         list item consist of ETag of attachment (page), attachment (page) ID
         and path to the attachment (page))

         An ETagDocument is used in two ways
         case 1) To hold information about the last mirroring
         case 2) To collect information about the current mirroring
    """

    ATTACHMENT = 'attachment'
    PAGE = 'page'


    def __init__(self):
        self.document=None
        self.attachment_map = {}
        self.page_map = {}

    def create(self):
        """
        The method is used in case 2. It creates a new document
        """
        self.document = minidom.Document()
        site_element = self.document.createElement('site')        
        self.document.appendChild(site_element)

        attachments_element = self.document.createElement('attachments')
        pages_element = self.document.createElement('pages')
        site_element.appendChild(attachments_element)
        site_element.appendChild(pages_element)


    def set_modification_time(self, time):
        """
        The method is used in case 2. It adds a timestamp of the current 
        mirroring to the document
        """
        self.document.getElementsByTagName('site')[0].setAttribute('updated', time)



    def check_attachment(self, id, etag):
        """
        The method is used in case 1. It checks if an attachment with ID=id and
        ETag=etag was added or modified since the last time. If not,
        the attachment is removed from the attachment_map.

        Args:
            id: The ID of the attachment to be checked
            etag: The ETag of the attachment to be checked
        Returns:
            True if the ETagDocument contains an attachment with ID=id and
            ETag=etag
            False otherwise
        """
        if self.attachment_map.has_key(id) and self.attachment_map[id][0] == etag:
            del self.attachment_map[id]
            return False
        else:
            return True


    def check_page(self, id, source):
        """
        The method is used in case 1. It checks if an page with ID=id
        source=source was added or moved since the last time. If not,
        the page is removed from the page_map.

        Args:
            id: The ID of the page to be checked
            source: The source of the page to be source
        """
        if self.page_map.has_key(id) and self.page_map[id][1] == source:
            del self.page_map[id]




    def get_unused_pages(self):
        """
        The method is used in case 1. Returns pages that weren't removed from
        the page_map while using check_page method

        Returns: List containing tuples of the form [etag, source]
        """
        return self.page_map.values()


    def get_unused_attachments(self):
        """
        The method is used in case 1. Returns attachments that weren't removed
        from the attachment_map while using check_attachment method

        Returns: List containing tuples of the form [etag, source]
        """
        return self.attachment_map.values()


    def add_element(self, id, source, etag, type):
        """
        The method is used in case 2. Adds an element to the XML document
        representing by the ETagDocument.
        The element consists of three attributes (id, source, etag) that hold
        information about either attachment or page (according to the type)
        Args:
            id: ID of the attachment (page)
            source: Path to the attachment (page)
            etag: ETag of the attachment (page)
            type: Type of the element, possible types are 'attachment' and 'page'
        """
        element = self.document.createElement(type)
        element.setAttribute('id', str(id))
        element.setAttribute('source', str(source))
        element.setAttribute('etag', str(etag))
        self.document.getElementsByTagName(self.get_parent_element(type))[0].appendChild(element)



    def get_parent_element(self, type):
        if type == self.ATTACHMENT:
            return 'attachments'
        elif type == self.PAGE:
            return 'pages'



    def get_modification_time(self):
        """
        The method is used in case 1. It gets a timestamp from the last
        mirroring
        """
        return self.document.getElementsByTagName('site')[0].getAttribute('updated')



    def get_document(self, path):
        """
        The method is used in case 1. It parses an XML document at the path and 
        procces it (ie. fill page_map and attachment_map)
        
        Args:
            path: A path to the etag XML document
        """
        self.document = minidom.parse(path)
        for node in self.document.getElementsByTagName('attachment'):
            self.attachment_map[node.getAttribute('id')]=[node.getAttribute('etag'),node.getAttribute('source')]
        for node in self.document.getElementsByTagName('page'):
            self.page_map[node.getAttribute('id')]=[node.getAttribute('etag'),node.getAttribute('source')]


        
    def to_string(self):
        return self.document.toprettyxml(encoding='UTF-8')



if __name__ == "__main__":
    pass
    


#!/usr/bin/python

import sys
#from distutils.core import setup
from setuptools import setup, find_packages
import sys, os

required = []

if sys.version_info[:3] < (2, 5, 0):
  required.append('elementtree')

setup(
    name='gsmirror',
    version='1.0.2',
    description='Google Sites Mirroring & Backup Script',
    long_description = """\
A Python module and script, which is able to download content from Google Sites (via Sites Data API) and create a mirror of the original web sites, but with a custom design (via Cheetah template) - for hosting on a traditional webserver.

Google Sites then become just an editing interface for the content, which is displayed somewhere else - with the possibility to completely customize the design of the websites.

The complete Sites structure is preserved, including the attachments. The script creates directories and files emulating the original Sites on the local disk. To publish such mirror of the Google Sites, it is enough to copy this directories via FTP to any webserver. We plan to support also direct hosting on the Google App Engine.
""",
    author='Jan Rychtar',
    author_email='honza.rychtar@gmail.com',
    license='New BSD License',
    url='http://code.google.com/p/google-sites-mirror/',
    packages=[ 'gsmirroring' ],
    package_dir = {'gsmirroring':'src/gsmirroring' },
    install_requires= [ 
         'Cheetah',
	 'BeautifulSoup',
         'gdata'
         ],
    package_data = {
        '': ['templates/*'],
        },
    entry_points = """
        [console_scripts]
        gsmirror = gsmirroring.gsmirror:main
    """

)
